/**
 * @fileoverview A simple organizer for ordering hooks.
 * @author Romullo @hiukky
 */
'use strict';
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DEFAULT_GROUPS = void 0;
exports.DEFAULT_GROUPS = [
    'useReducer',
    'useContext',
    'useState',
    'useRef',
    'useDispatch',
    'useCallback',
    'useEffect',
];
module.exports = {
    meta: {
        docs: {
            description: 'A simple organizer for ordering hooks.',
            category: 'Non-matching declaration order.',
            url: 'https://github.com/hiukky/eslint-plugin-hooks/blob/main/docs/rules/sort.md',
            recommended: false,
        },
        fixable: undefined,
        schema: [
            {
                type: 'object',
                properties: {
                    groups: {
                        type: 'array',
                    },
                },
            },
        ],
    },
    create: function (ctx) {
        var options = ctx.options[0];
        var groups = (options === null || options === void 0 ? void 0 : options.groups) || exports.DEFAULT_GROUPS;
        return {
            Program: function (_a) {
                var body = _a.body;
                body
                    .filter(function (_a) {
                    var type = _a.type;
                    return [
                        'FunctionDeclaration',
                        'VariableDeclaration',
                        'ExportNamedDeclaration',
                        'ExportDefaultDeclaration',
                    ].includes(type);
                })
                    .map(function (node) {
                    var _a, _b, _c;
                    var declarations = node;
                    var isExportableDeclaration = function () {
                        return ['ExportNamedDeclaration', 'ExportDefaultDeclaration'].includes(node.type);
                    };
                    if (isExportableDeclaration()) {
                        declarations =
                            ((_a = node['declaration']['declarations']) === null || _a === void 0 ? void 0 : _a[0]['init']) ||
                                node['declaration'];
                    }
                    else {
                        declarations = ((_b = node['declarations']) === null || _b === void 0 ? void 0 : _b[0]['init']) || node;
                    }
                    return (_c = declarations['body']) === null || _c === void 0 ? void 0 : _c['body'];
                })
                    .filter(Boolean)
                    .forEach(function (declarations) {
                    var nodes = [];
                    declarations.forEach(function (node) {
                        if (node['type'] === 'ExpressionStatement') {
                            nodes.push(node['expression']);
                        }
                        if (node['type'] === 'VariableDeclaration') {
                            nodes.push.apply(nodes, node['declarations']);
                        }
                    });
                    var hooks = nodes
                        .map(function (_a) {
                        var type = _a.type, callee = _a.callee, init = _a.init;
                        return (type === 'CallExpression'
                            ? [type, callee]
                            : type === 'VariableDeclarator'
                                ? [type, init]
                                : []);
                    })
                        .filter(function (node) { return node.length; })
                        .map(function (_a) {
                        var type = _a[0], declaration = _a[1];
                        var getHookName = function () {
                            return type === 'CallExpression'
                                ? declaration.name
                                : declaration.callee.name;
                        };
                        var getHookNode = function () {
                            return type === 'CallExpression' ? declaration : declaration.callee;
                        };
                        return [getHookName(), getHookNode()];
                    })
                        .filter(function (_a) {
                        var name = _a[0];
                        return (name === null || name === void 0 ? void 0 : name.slice(0, 3)) === 'use' && groups.includes(name);
                    });
                    var correctOrdering = __spreadArray([], hooks, true).sort(function (a, b) { return groups.indexOf(a[0]) - groups.indexOf(b[0]); });
                    hooks.forEach(function (hook, index) {
                        var noMatching = function () {
                            return correctOrdering.length > 1 &&
                                correctOrdering[index][0] !== hook[0];
                        };
                        if (noMatching()) {
                            ctx.report(hook[1], "Non-matching declaration order. " + hook[0] + " comes " + (!index ? 'after' : 'before') + " " + correctOrdering[index][0] + ".");
                        }
                    });
                });
            },
        };
    },
};
